﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using Newtonsoft.Json;

namespace TestConsole
{
    public class Program
    {
        public static void Main(string[] args)
        {
            Console.WriteLine("Client Credentials");
            TestClientCredentials();

            Console.WriteLine("\r\n\r\nResource Owner Password");
            TestResourceOwnerPassword();

            Console.ReadKey();
        }

        /// <summary>
        /// resource owner password credentials模式
        /// </summary>
        public static void TestResourceOwnerPassword()
        {
            var client = new HttpClientHepler("http://localhost:4537/connect/token");
            string accessToken = null, refreshToken = null;
            //获取AccessToken
            client.PostAsync(null,
                "grant_type=" + "password" +
                "&username=" + "admin" +
                "&password=" + "123" +
                "&client_id=" + "pwd_client" +
                "&client_secret=" + "pwd_secret" +
                "&scope=" + "api1 offline_access",  //scope需要用空格隔开，offline_access为获取RefreshToken
                hd => hd.ContentType = new MediaTypeHeaderValue("application/x-www-form-urlencoded"),
                rtnVal => 
                {
                    var jsonVal = JsonConvert.DeserializeObject<dynamic>(rtnVal);
                    accessToken = jsonVal.access_token;
                    refreshToken = jsonVal.refresh_token;
                },
                fault => Console.WriteLine(fault),
                ex => Console.WriteLine(ex)).Wait();

            if (!string.IsNullOrEmpty(refreshToken))
            {
                //使用RefreshToken获取新的AccessToken
                client.PostAsync(null,
                    "grant_type=" + "refresh_token" +
                    "&client_id=" + "pwd_client" +
                    "&client_secret=" + "pwd_secret" +
                    "&refresh_token=" + refreshToken,
                    hd => hd.ContentType = new MediaTypeHeaderValue("application/x-www-form-urlencoded"),
                    rtnVal => Console.WriteLine("refresh之后的结果: \r\n" + rtnVal),
                    fault => Console.WriteLine(fault),
                    ex => Console.WriteLine(ex)).Wait();
            }

            if (!string.IsNullOrEmpty(accessToken))
            {
                //访问资源服务
                client.Url = "http://localhost:4823/api/values";
                client.GetAsync(null,
                        hd => hd.Add("Authorization", "Bearer " + accessToken),
                        rtnVal => Console.WriteLine("\r\n访问资源服: \r\n" + rtnVal),
                        fault => Console.WriteLine(fault),
                        ex => Console.WriteLine(ex)).Wait(); 
            }
        }

        /// <summary>
        /// client credentials模式
        /// </summary>
        public static void TestClientCredentials()
        {
            var client = new HttpClientHepler("http://localhost:4537/connect/token");
            string accessToken = null;
            //获取AccessToken
            client.PostAsync(null,
                "grant_type=" + "client_credentials" +
                "&client_id=" + "credt_client" +
                "&client_secret=" + "credt_secret" +
                "&scope=" + "api1",  //不要加上offline_access，因为Client Credentials模式不支持RefreshToken的，不然会授权失败
                hd => hd.ContentType = new MediaTypeHeaderValue("application/x-www-form-urlencoded"),
                rtnVal =>
                {
                    var jsonVal = JsonConvert.DeserializeObject<dynamic>(rtnVal);
                    accessToken = jsonVal.access_token;
                },
                fault => Console.WriteLine(fault),
                ex => Console.WriteLine(ex)).Wait();

            if (!string.IsNullOrEmpty(accessToken))
            {
                //访问资源服务
                client.Url = "http://localhost:4823/api/values";
                client.GetAsync(null,
                        hd => hd.Add("Authorization", "Bearer " + accessToken),
                        rtnVal => Console.WriteLine("访问资源服: \r\n" + rtnVal),
                        fault => Console.WriteLine(fault),
                        ex => Console.WriteLine(ex)).Wait();
            }
        }

    }
}
